"""
Computation of transcendental functions.
"""

#*****************************************************************************
#       Copyright (C) 2004 William Stein <was@math.harvard.edu>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#    This code is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    General Public License for more details.
#
#  The full text of the GPL is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************

import sage.pari.py_pari
import sage.rings.complex_number

def __prep_num(x):
    if isinstance(x, sage.rings.complex_number.ComplexNumber):
        x = str(x).replace("i","I")
    else:
        x = str(x)
    return x

I = sage.rings.complex_field.ComplexField().gen(0)
def __eval(x):
    return eval(x)

def zeta(s):
    """
    Riemann zeta function at s with s a real, complex or p-adic number.
    EXAMPLES:
        >>> zeta(2)
        1.6449340668482264
    """
    s = "zeta(%s)"%__prep_num(s)
    return __eval(sage.pari.py_pari.eval(s))


# doctests
if __name__ ==  '__main__':
    import doctest, sys
    doctest.testmod(sys.modules[__name__])
