"""
pari_pyx.pyx
"""

import sage.misc.misc

include 'cdefs.pxi'

cdef extern from 'pari/pari.h':
    ctypedef long* GEN
    GEN     flisseq(char *t)
    char*   GENtostr(GEN x)
    void    cgiv(GEN x)
    ctypedef int size_t
    void    pari_init(size_t parisize, unsigned long maxprime)
    void    pari_sig_init(...)

    GEN     flisexpr(char *t)

cdef extern from 'setjmp.h':
    struct __jmp_buf_tag:
        pass
    ctypedef __jmp_buf_tag jmp_buf
    int setjmp (jmp_buf __env)

cdef extern from 'pari/anal.h':
    struct anal_xxx:
        jmp_buf env
    ctypedef anal_xxx gp_data
    extern gp_data* GP_DATA

# Initialize the relevant part of GP for exception trapping.
cdef gp_data __GP_DATA
GP_DATA = &__GP_DATA

initialized = False

def init(long size=100000000, unsigned long maxprime=50000):
    """
    Initialize the PARI system.
    
    INPUT:
        size -- long, the number of bytes given to PARI to work with
                (default: 100000000)
        maxprime -- unsigned long, upper limit on a precomputed prime
                    number table  (default: 50000)
    
    If you do not want prime numbers, put maxprime=2, but be careful
    because many PARI functions require this table.  If you get the
    error message "not enough precomputed primes", increase this
    parameter.
    EXAMPLES:
        >>> pari_init()   
    """
    global initialized
    if not initialized:
        pari_init(size, maxprime);
        initialized = True
    else:
        raise RuntimeError, "pari_init has already been called."
#init()

def execute(s):
    #sage.misc.misc.verbose("Pari command: "+s)
    if not initialized:
        init()
    # Parse s in PARI.
    s = s + "\n" 
    flisseq(s)
    
def eval(s):
    #sage.misc.misc.verbose("Pari command: "+s)
    if not initialized:
        init()
    cdef GEN x
    cdef char *c
    # Parse s in pari and evaluate result.
    s = s + "\n"
    x = flisseq(s)
    # Get result as a C-string
    c = GENtostr(x)
    # De-callocate memory used by x:
    cgiv(x)
    # Convert c to a Python string.
    s = str(c)
    # Free memory used by c
    free(c)
    return s
    

